package com.rshy.project.hy.manager.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.rshy.project.hy.manager.RshyJkManager;
import com.rshy.project.hy.model.constant.ItfwConstant;
import com.rshy.project.hy.model.enums.HyTypeEnum;
import com.rshy.project.hy.model.enums.PassStatusEnum;
import com.rshy.project.hy.model.enums.RepositoryEnum;
import com.rshy.project.hy.model.enums.SexEnum;
import com.rshy.project.hy.server.FaceRetrievaServer;
import com.rshy.project.hy.server.ItfwServer;
import com.rshy.project.hy.server.dto.PctjkmInfoDTO;
import com.rshy.project.hy.server.dto.PctrchlDTO;
import com.rshy.project.hy.server.dto.RepositoryDTO;
import com.rshy.project.hy.server.dto.ResultDTO;
import com.rshy.project.hy.server.param.RshyParam;
import com.rshy.project.hy.server.param.RshyQuickParam;
import com.rshy.project.hy.server.vo.RetrievaResultVO;
import com.rshy.project.hy.server.vo.RetrievalRepositoryVO;
import com.rshy.project.hy.server.vo.RshyVo;
import com.rshy.project.hy.util.ObjectUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author: lileilei
 * @date: 2022/4/23 20:12
 * @description:
 */
@Component
@Slf4j
public class RshyJkManagerImpl implements RshyJkManager {
    @Autowired
    private FaceRetrievaServer faceRetrievaServer;
    @Autowired
    private ItfwServer itfwServer;
    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Override
    public RshyVo getRshyInfo(RshyParam rshyParam) {
        // timeout 转成毫秒
        rshyParam.setTimeout((rshyParam.getTimeout() - 1) * 1000);
        log.info("普通核验接口请求参数：{}", JSON.toJSONString(rshyParam));
        RshyVo rshyVo = new RshyVo();
        ResultDTO resultDTO = new ResultDTO();
        if (StringUtils.isNotBlank(rshyParam.getBarCode())) {
            DateTime beginDate = DateTime.now();
            PctjkmInfoDTO barCodes = null;
            try {
                //查询健康码信息
                barCodes = itfwServer.getPctjkm(rshyParam.getBarCode(), rshyParam.getTimeout());
            } catch (Exception e) {
                log.error("查询健康码信息异常:{}", e.getMessage());
                e.printStackTrace();
            }

            if (barCodes == null) {
                return rshyVo;
            }

            rshyParam.setSfzh(barCodes.getId_num());
            rshyParam.setXm(barCodes.getUser_name());
            String codeLabel = barCodes.getCOLOR_CODE_LABEL();
            String phone = barCodes.getPn();
            rshyVo.setXm(rshyParam.getXm()).setSfzh(rshyParam.getSfzh()).setHealthCode(codeLabel).setPhone(phone);

            if (PassStatusEnum.RED_CODE.getDesc().equalsIgnoreCase(barCodes.getCOLOR_CODE_LABEL())) {
                rshyVo.setPass(PassStatusEnum.RED_CODE.getCode()).setTimes(times(rshyParam.getSfzh())).setTts(PassStatusEnum.RED_CODE.getDesc()).setDisplayMsg(PassStatusEnum.RED_CODE.getDesc()).setWarning(PassStatusEnum.RED_CODE.getDesc()).setKind("健康码" + PassStatusEnum.RED_CODE.getDesc());
                return rshyVo;
            } else if (PassStatusEnum.YELLOW_CODE.getDesc().equalsIgnoreCase(barCodes.getCOLOR_CODE_LABEL())) {
                rshyVo.setPass(PassStatusEnum.YELLOW_CODE.getCode()).setTimes(times(rshyParam.getSfzh())).setTts(PassStatusEnum.YELLOW_CODE.getDesc()).setDisplayMsg(PassStatusEnum.YELLOW_CODE.getDesc()).setWarning(PassStatusEnum.YELLOW_CODE.getDesc()).setKind("健康码" + PassStatusEnum.YELLOW_CODE.getDesc());
                return rshyVo;
            } else {
                rshyVo.setTts(barCodes.getCOLOR_CODE_LABEL());
            }
            rshyParam.setTimeout((int) (rshyParam.getTimeout() - DateUtil.between(beginDate, DateTime.now(), DateUnit.MS)));
        }

        List<Runnable> taskList = new ArrayList<Runnable>() {
            {
                add(() -> resultDTO.setPctdfkDTO(itfwServer.getPctdfk(rshyParam.getSfzh(), "", rshyParam.getTimeout())));
                add(() -> resultDTO.setPctjsryDTO(itfwServer.getPctjsry(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctztDTO(itfwServer.getPctzt(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctzfbaDTO(itfwServer.getPctzfba(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctqkryDTO(itfwServer.getPctqkry(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctxdryDTO(itfwServer.getPctxdry(rshyParam.getSfzh(), "", rshyParam.getTimeout())));
                add(() -> resultDTO.setPctzdryDTO(itfwServer.getPctzdry(rshyParam.getSfzh(), rshyParam.getTimeout())));
                add(() -> resultDTO.setPctrchlDTO(itfwServer.getPctrchl(rshyParam.getSfzh(), rshyParam.getTimeout())));
            }
        };

        //异步流并发处理
        taskList.parallelStream().forEach(v -> {
            try {
                v.run();
            } catch (Exception e) {
                log.error("异常信息：{}", e.getMessage());
                e.printStackTrace();
            }
        });

        log.info("IT服务平台接口调用完成");
        rshyVo.setXm(rshyParam.getXm()).setSfzh(rshyParam.getSfzh()).setTimes(times(rshyParam.getSfzh()));
        rshyVo.setXb(SexEnum.codeOf(IdcardUtil.getGenderByIdCard(rshyParam.getSfzh())).getDesc());
        Date date = DateUtil.parse(IdcardUtil.getBirthByIdCard(rshyParam.getSfzh()));
        rshyVo.setBirthday(DateUtil.format(date, "yyyy-M-d"));

        PctrchlDTO pctrchlDTO = resultDTO.getPctrchlDTO();
        resultDTO.setPctrchlDTO(null);
        String hcjg = "";
        if (pctrchlDTO != null) {
            hcjg = pctrchlDTO.getHcjg();
        }

        if (resultDTO.getPctztDTO() != null || StrUtil.containsAnyIgnoreCase(hcjg, "抓捕")) {
            rshyVo.setPass(PassStatusEnum.NO_PASSING.getCode()).setTts(PassStatusEnum.NO_PASSING.getDesc()).setDisplayMsg(PassStatusEnum.NO_PASSING.getDesc()).setWarning(new StringBuilder().append("在逃人员:").append(resultDTO.getPctztDTO().getJyaq()).toString()).setKind(resultDTO.getPctztDTO().getZtrylxmc());
        } else if (ObjectUtils.allFieldIsNULL(resultDTO) || StrUtil.containsAnyIgnoreCase(hcjg, "通过")) {
            rshyVo.setPass(PassStatusEnum.NORMAL_TRAFFIC.getCode()).setDisplayMsg(PassStatusEnum.NORMAL_TRAFFIC.getDesc()).setTts(PassStatusEnum.NORMAL_TRAFFIC.getDesc());
        } else {
            rshyVo.setPass(PassStatusEnum.RIGHT_WAY.getCode()).setKind(kind(resultDTO)).setWarning(warning(resultDTO)).setDisplayMsg(PassStatusEnum.RIGHT_WAY.getDesc()).setTts(PassStatusEnum.RIGHT_WAY.getDesc());
        }
        if (rshyVo.getHealthCode() != null) {
            rshyVo.setTts(rshyVo.getHealthCode());
        }

        log.info("普通核验响应内容：{}", JSON.toJSONString(rshyVo));
        return rshyVo;
    }

    @Override
    public RshyVo getQuickRyhyInfo(RshyQuickParam rshyQuickParam) {
        RshyVo rshyVo = new RshyVo();
        RetrievaResultVO retrievaResultVO = faceRetrievaServer.retrieval(rshyQuickParam.getPhoto(), rshyQuickParam.getTimeout());
        List<RetrievalRepositoryVO> voList = Convert.toList(RetrievalRepositoryVO.class, retrievaResultVO.getResults());
        if (CollectionUtil.isEmpty(voList)) {
            rshyVo.setPass(PassStatusEnum.NORMAL_TRAFFIC.getCode());
            rshyVo.setDisplayMsg(PassStatusEnum.NORMAL_TRAFFIC.getDesc());
            rshyVo.setTts(PassStatusEnum.NORMAL_TRAFFIC.getDesc());
            return rshyVo;
        }

        //查询人像库信息
        RetrievaResultVO repository = faceRetrievaServer.repository(rshyQuickParam.getTimeout());
        List<RepositoryDTO> dtoList = Convert.toList(RepositoryDTO.class, repository.getResults());
        Map<String, String> stringMap = CollectionUtil.isNotEmpty(dtoList) ? dtoList.stream().collect(Collectors.toMap(RepositoryDTO::getId, RepositoryDTO::getName, (v1, v2) -> v1)) : MapUtil.newHashMap();
        RetrievalRepositoryVO repositoryVO = voList.stream().findFirst().get();
        rshyVo.setPass(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getCode() : PassStatusEnum.RIGHT_WAY.getCode()).setKind(StrUtil.EMPTY)
                .setWarning(stringMap.get(repositoryVO.getRepository_id())).setDisplayMsg(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getDesc() : PassStatusEnum.RIGHT_WAY.getDesc())
                .setTts(RepositoryEnum.COUNTRY_LARGE.getId().equals(repositoryVO.getRepository_id()) ? PassStatusEnum.NO_PASSING.getDesc() : PassStatusEnum.RIGHT_WAY.getDesc()).setHealthCode(StrUtil.EMPTY).setSfzh(repositoryVO.getPersonId())
                .setXm(repositoryVO.getName()).setXb(SexEnum.codeOf(repositoryVO.getGender()).getDesc()).setMz(StrUtil.EMPTY).setPhone(StrUtil.EMPTY).setAddress(StrUtil.EMPTY).setPhoto(rshyQuickParam.getPhoto());
        log.info("快速核验响应内容：{}", JSON.toJSONString(rshyVo));
        return rshyVo;
    }

    /**
     * 今日进站次数
     *
     * @param idNumber 身份证号
     * @return
     */
    private Integer times(String idNumber) {
        String cacheKey = new StringBuilder().append(ItfwConstant.TIMES_KEY).append(idNumber).toString();
        Integer times = 1;
        String timesStr = stringRedisTemplate.opsForValue().get(cacheKey);
        if (timesStr == null) {
            //今日进站次数
            stringRedisTemplate.opsForValue().set(cacheKey, String.valueOf(times), DateUtil.endOfDay(DateUtil.date()).getTime() - DateUtil.date().getTime(), TimeUnit.MILLISECONDS);
        } else {
            times = Integer.valueOf(timesStr) + 1;
            stringRedisTemplate.opsForValue().increment(cacheKey);
        }

        return times;
    }

    /**
     * 人员类型简要概述
     *
     * @param resultDTO
     * @return
     */
    private String kind(ResultDTO resultDTO) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(resultDTO.getPctdfkDTO() != null ? HyTypeEnum.pctdfk.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctjsryDTO() != null ? HyTypeEnum.pctjsry.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctxdryDTO() != null ? HyTypeEnum.pctxdry.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctqkryDTO() != null ? HyTypeEnum.pctqkry.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctjkmInfoDTO() != null ? HyTypeEnum.pctjkm.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctzfbaDTO() != null ? HyTypeEnum.pctzfba.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctzdryDTO() != null ? HyTypeEnum.pctzdry.getDesc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctrchlDTO() != null ? resultDTO.getPctrchlDTO().getRylb() : StrUtil.EMPTY);
        return stringBuilder.toString();
    }

    /**
     * 警示信息
     *
     * @param resultDTO
     * @return
     */
    private String warning(ResultDTO resultDTO) {
        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append(resultDTO.getPctdfkDTO() != null ? resultDTO.getPctdfkDTO().getJyaq() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctjsryDTO() != null ? resultDTO.getPctjsryDTO().getJyaq() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctxdryDTO() != null ? resultDTO.getPctxdryDTO().getXdcsmc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctqkryDTO() != null ? resultDTO.getPctqkryDTO().getLsqkqk() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctzfbaDTO() != null ? resultDTO.getPctzfbaDTO().getAjmc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctzdryDTO() != null ? resultDTO.getPctzdryDTO().getAjlbmc() : StrUtil.EMPTY);
        stringBuilder.append(resultDTO.getPctrchlDTO() != null ? resultDTO.getPctrchlDTO().getBjxx() : StrUtil.EMPTY);
        return stringBuilder.toString();
    }
}
